@@ -2,7 +2,7 @@
 import string, random
 
 from flask import Blueprint, current_app
-from flask import render_template, url_for, flash, redirect, request
+from flask import render_template, url_for, flash, redirect, request, abort
 from mortimer import bcrypt
 from mortimer.forms import (
     RegistrationForm,
@@ -15,7 +15,7 @@ from mortimer.models import User, WebExperiment
 from mortimer.utils import send_reset_email, create_fernet
 from flask_login import login_user, current_user, logout_user, login_required
 
-
+# pylint: disable=no-member
 users = Blueprint("users", __name__)
 
 
@@ -28,22 +28,17 @@ def register():
     form = RegistrationForm()
 
     if form.validate_on_submit():
+        if User.objects(username=form.username.data):
+            flash("Username already taken. Please choose a different one.", "error")
+            return redirect(url_for("users.register"))
+        elif User.objects(email=form.email.data):
+            flash("Email already taken. Please choose a different one.", "error")
+            return redirect(url_for("users.register"))
+
         hashed_password = bcrypt.generate_password_hash(form.password.data).decode("utf-8")
         user = User(username=form.username.data, email=form.email.data, password=hashed_password)
         user.encryption_key = User.generate_encryption_key()
-
-        # create user for alfred database and save credentials in user object
-        user_lower = user.username.lower().replace(" ", "_")
-        user.alfred_user = "alfredUser_{}".format(user_lower)
-        if User.objects(alfred_user=user.alfred_user):  # pylint: disable=no-member
-            flash("Username already taken. Please choose a different one.", "error")
-            return redirect(url_for("users.register"))
-        user.alfred_pw = User.generate_password()  # password is encrypted
-        user.alfred_col = "col_{}".format(user_lower)
-        user.create_db_user()
-
-        # create user for local experiments
-        user.prepare_local_db_user()
+        user.set_db_config()
 
         # save user
         user.save()
@@ -118,23 +113,8 @@ def account():
         form.username.data = current_user.username
         form.email.data = current_user.email
 
-    if not current_user.local_db_user:
-        current_user.prepare_local_db_user()
-        current_user.save()
-
-    f = create_fernet()
-    local_pw = f.decrypt(current_user.local_db_pw).decode()
-    user_key = f.decrypt(current_user.encryption_key).decode()
-
-    info = {
-        "username": ("Username for local DB", current_user.local_db_user),
-        "password": ("Password for local DB", local_pw),
-        "col": ("Collection for local DB", current_user.local_col),
-        "_key": ("Encryption Key", user_key),
-    }
-
     return render_template(
-        "account.html", title="Account", form=form, user=current_user, info=info
+        "account.html", title="Account", form=form, user=current_user
     )
 
 
